// database.cpp -- MFC DAO Database specific functions

#include "stdafx.h"
#include "database.h"

/////////////////////////////////////////////////////////////////////////////
// closeDatabase()
// close the database and destruct it
// IN/OUT:: ppDatabase - pointer to pointer to database to close and
// destruct

void closeDatabase(CDaoDatabase **ppDatabase)
{
	// only process if the database object exists
	if (*ppDatabase != NULL)
	{
		if ((*ppDatabase)->IsOpen())
			(*ppDatabase)->Close();

		// closing doesn't delete the object
		delete *ppDatabase;
		*ppDatabase = NULL;
	}
}

/////////////////////////////////////////////////////////////////////////////
// openDatabase()
// open the database from a file (close if necessary)
// IN/OUT: ppDatabase - pointer to pointer to database object
// IN: fileName - name of .mdb file to open
// IN: bReportNoOpen - TRUE by default, if true, if open fails
//     because the specified MDB file doesn't exist, report that
//     fact to the user.  Note: other errors are reported always
// RETURN: 1 if success, 0 if non-fatal failure, -1 if fatal failure

int openDatabase(CDaoDatabase **ppDatabase, CString fileName,
				  BOOL bReportNoOpen /* = TRUE */)
{
	// initialize success indicator
	int nReturnCode = 1;

	// close and delete if necessary
	if (*ppDatabase != NULL)
	{
		if ((*ppDatabase)->IsOpen())
			closeDatabase(ppDatabase);
		delete *ppDatabase;
	}

	// construct new database
	*ppDatabase = new CDaoDatabase;

	// failed to allocate
	if (ppDatabase == NULL)
		return -1; // fatal error

	// now open the database object with error checking
	try
	{
		AfxGetModuleState()->m_dwVersion = 0x0601;
		(*ppDatabase)->Open(fileName);
	}
	catch (CDaoException *e)
	{
		// special case--couldn't find the file, so it may be because
		// user specified a new file to open
		if (e->m_pErrorInfo->m_lErrorCode == 3024)
		{
			if (bReportNoOpen)
			{
				// create a message to display
				CString message = _T("Couldn't open database--Exception: ");
				message += e->m_pErrorInfo->m_strDescription;

				// output status
				AfxMessageBox(message);
			}

			// indicate failure but not fatal
			nReturnCode = 0;
		}
		else // other type of DAO exception--always report
		{
			// create a message to display
			CString message = _T("Couldn't open database--Exception: ");
			message += e->m_pErrorInfo->m_strDescription;

			// output status
			AfxMessageBox(message);

			// indicate fatal error
			nReturnCode = -1;
		}

		// not rethrowing, so delete exception
		e->Delete();

		delete *ppDatabase;
		*ppDatabase = NULL;
	}
	catch (CMemoryException *e)
	{
		// output status
		AfxMessageBox(_T("Failed to open database--Memory exception thrown."));

		// not rethrowing, so delete exception
		e->Delete();

		delete *ppDatabase;
		*ppDatabase = NULL;

		// indicate fatal error
		nReturnCode = -1;
	}

	return nReturnCode;
}

/////////////////////////////////////////////////////////////////////////////
// getTableInfo()
// fill a tableinfo struct - handle exceptions
// IN: pDatabase - pointer to database object whose tabledef collection we will
//     access
// OUT: pTableInfo - pointer to structure to hold the tabledef info
// IN: tableIndex - index into the collection indicating which item we want
// IN: bReportErrors - TRUE by default, if TRUE, report all errors, else silent
// RETURN: TRUE if information obtained, FALSE otherwise

BOOL getTableInfo(CDaoDatabase *pDatabase, CDaoTableDefInfo *pTableInfo,
				  int tableIndex, BOOL bReportErrors /*= TRUE*/)
{
	// if the database is non-existent, then the answer is obvious
	if (pDatabase == NULL)
		return FALSE;

	// initialize success indicator
	BOOL bSuccess = TRUE;

	TRY
	{
		// try to get info on the table
		pDatabase->GetTableDefInfo(tableIndex, *pTableInfo, AFX_DAO_ALL_INFO );
	}
	CATCH (CDaoException, e)
	{
		// construct a meaningful message if request
		if (bReportErrors)
		{
			CString strMessage = _T("Couldn't get information on table--Exception: ");
			strMessage += e->m_pErrorInfo->m_strDescription;

			AfxMessageBox(strMessage);
		}

		// indicate failure
		bSuccess = FALSE;
	}
	AND_CATCH (CMemoryException, e)
	{
		// output status if requested
		if (bReportErrors)
			AfxMessageBox(_T("Failed to get info on table--Memory exception thrown."));

		// indicate failure
		bSuccess = FALSE;
	}
	END_CATCH

	// return status
	return bSuccess;
}
